love.graphics.rcircle = function( mode, x, y, radius )
	love.graphics.rectangle( mode, x-radius, y-radius, radius*2, radius*2)
end

local slime = {}

function slime.new(x, y, grid_size, slime_size)
	local factor = 2*grid_size/100
	
	slime.grid_size = grid_size
	slime.slime_size = slime_size
	
	slime.r = grid_size*slime_size

	slime.g = 1000*factor
--	slime.jump_vy = 500*factor^0.5
	slime.jump_vy = 380*factor^0.5
--	slime.vx_max=230*factor^0.5
	slime.vx_max=170*factor^0.5
	slime.ax_max=(1000*grid_size)
	slime.wall_speed = (1/2)*grid_size*factor
	
	slime.x=x slime.y=y
	slime.vx=0 slime.vy=0

	slime.on_floor = false
	slime.on_ceiling = false
	slime.on_wall = false
end

function is_tile_wall (i, j)
	return map[j] and map[j][i] and map[j][i] == 1 or not (map[j] and map[j][i])
end

function get_grid (x, y)
	return math.floor(x/grid_size+0.5), math.floor(y/grid_size+0.5)
end

function bounce_floor (dt)
	local bevel = 0.8*slime.r
	-- dynamic solution
	local i1d, j1d = get_grid (slime.x+dt*slime.vx-bevel-0.00001, slime.y+slime.r+dt*slime.vy)
	local i2d, j2d = get_grid (slime.x+dt*slime.vx, slime.y+slime.r+dt*slime.vy)
	local i3d, j3d = get_grid (slime.x+dt*slime.vx+bevel, slime.y+slime.r+dt*slime.vy)
	if is_tile_wall (i2d, j2d) and not slime.on_ceiling then
		slime.y = (j2d-0.5)*grid_size-slime.r
		slime.vy = 0
		slime.on_floor = true
--	elseif is_tile_wall (i1d, j1d) and slime.vx > 0 then
--		slime.vx = math.max (slime.vy, slime.vx)
--		slime.vy = slime.vx
--	elseif is_tile_wall (i3d, j3d) and slime.vx < 0 then
--		slime.vx = math.min (slime.vy, slime.vx)
--		slime.vy = -slime.vx
	else
		slime.on_floor = false
	end
end

function bounce_ceiling (dt)
	-- actual position
	local i, j = get_grid (slime.x, slime.y)
	-- static solution
	local i8, j8 = get_grid (slime.x, slime.y-slime.r-0.00001)
	-- dynamic solution
	local i8d, j8d = get_grid (slime.x+dt*slime.vx, slime.y-slime.r+dt*slime.vy-0.00001)
	if (not slime.on_ceiling) and is_tile_wall (i8d, j8d) then
		slime.y = (j-0.5)*grid_size+slime.r
		slime.vy = 0
		slime.on_ceiling = true
	elseif slime.on_ceiling and is_tile_wall (i8, j8) then
		-- idle on ceiling
		slime.vy = 0
	else
		slime.on_ceiling = false
	end
end

function bounce_wall (dt)
	
	
	
	if slime.vx == 0 then 
		local i4, j4 = get_grid (slime.x-slime.r-0.00001, slime.y)
		local i6, j6 = get_grid (slime.x+slime.r, slime.y)
		if slime.on_wall and not (is_tile_wall (i4, j4) or is_tile_wall (i6, j6)) then
			slime.on_wall = false
		elseif slime.vy>slime.wall_speed then
			if not slime.on_wall and is_tile_wall (i4, j4) then
				-- left
				slime.x = (i4+0.5)*grid_size+slime.r
				slime.on_wall = true
			elseif not slime.on_wall and is_tile_wall (i6, j6) then
				-- right
				slime.x = (i6-0.5)*grid_size-slime.r
				slime.on_wall = true
			end
		end
		return 
	end
	local sign = slime.vx > 0 and 1 or slime.vx < 0 and -1 or 0
	
	-- current cell
	local i, j = get_grid (slime.x, slime.y)
	--- dynamic solution: 
	local i2d, j2d = get_grid (slime.x+sign*slime.r+dt*slime.vx+sign*0.00001, slime.y+dt*slime.vy)

--	if is_tile_wall (i2d, j2d) and slime.vy>0 then
	if is_tile_wall (i2d, j2d) then
		slime.vx = 0
		slime.x = (i+sign*0.5)*grid_size-sign*slime.r
		if slime.vy >= 0 then
			slime.on_wall = true
		end
	else
		slime.on_wall = false
	end

end





function slime.update(dt, map)
	
	if love.keyboard.isDown('d') then
		slime.vx = slime.vx + dt*slime.ax_max
		slime.vx = math.min (slime.vx, slime.vx_max)
	elseif love.keyboard.isDown('a') then
		slime.vx = slime.vx - dt*slime.ax_max
		slime.vx = math.max (slime.vx, -slime.vx_max)
	elseif love.keyboard.isDown('m') then
		slime.x = slime.x + grid_size
		return
	elseif not (slime.vx == 0) then
		local sign = slime.vx > 0 and 1 or -1
		slime.vx = sign * math.max (0, math.abs(slime.vx) - dt*slime.ax_max)
	end

	if slime.on_wall and not (slime.on_ceiling or slime.on_floor) then
		slime.vy = slime.wall_speed
	elseif not slime.on_corner then
		slime.vy = slime.vy + dt*slime.g
	end


--	check_corners (dt)
	if false and slime.on_corner then
--		bounce_corner (dt)
	else
		bounce_floor (dt)
		bounce_ceiling (dt)
		bounce_wall (dt)
	end
	
	slime.x = slime.x + dt*slime.vx
	slime.y = slime.y + dt*slime.vy
end

function slime.draw()
	if slime.on_corner then
		love.graphics.setColor(0.5,0,0)
	
	elseif slime.on_ceiling then
		love.graphics.setColor(0,0,0.7)
	elseif slime.on_wall then
		love.graphics.setColor(0,0.5,0)
	else
		love.graphics.setColor(0,0,0)
	end
	love.graphics.rcircle('fill', slime.x,slime.y, slime.r)
	love.graphics.setColor(1,1,1)
	love.graphics.rcircle('line', slime.x,slime.y, slime.r)
	
--	local ii = 1
--	for i, v in pairs (slime) do
--		if type (v) ~= "table" and type (v) ~= "function" then
--			if type (v) == "boolean" and v then
--				love.graphics.setColor(1,1,1)
--			else
--				love.graphics.setColor(0,1,0)
--			end
--			love.graphics.print(i..': '..tostring(v), 1400,ii*20)
--			ii=ii+1
--		end
--	end
end

function slime.keypressed(key, scancode, isrepeat)
	if false then
		
	elseif key == "w" or key == "space" then
		if slime.on_floor or slime.on_wall or slime.on_corner then
			if slime.on_wall then
				slime.vy = -1.4*(slime.jump_vy)
			else
				slime.vy = -(slime.jump_vy)
			end
			
			slime.on_floor = false
			slime.on_wall = false
			slime.on_corner = false
		end
	elseif key == "s" then
	
		if slime.on_ceiling then
--			slime.vy = slime.vy
			slime.on_ceiling = false
		elseif slime.on_wall and (not slime.ignore_wall) then
			slime.ignore_wall = true
		end
	end
end

function slime.mousereleased( x, y, button, istouch, presses )
	slime.x = x
	slime.y = y
	slime.vy = 0
	
	slime.on_ceiling = false
	slime.on_floor = false
	slime.on_wall = false
end



return slime